@@ -2,7 +2,7 @@ package DBIx::Skinny;
 use strict;
 use warnings;
 
-our $VERSION = '0.0726';
+our $VERSION = '0.0727';
 
 use DBI;
 use DBIx::Skinny::Iterator;
@@ -44,7 +44,7 @@ sub import {
     my $schema = $opt{schema} || "$caller\::Schema";
 
     my $dbd_type = _dbd_type($connect_info);
-    my $_attribute = +{
+    my $_attributes = +{
         check_schema    => defined $connect_info->{check_schema} ? $connect_info->{check_schema} : 1,
         dsn             => $connect_info->{dsn},
         username        => $connect_info->{username},
@@ -65,7 +65,8 @@ sub import {
     {
         no strict 'refs';
         push @{"${caller}::ISA"}, $class;
-        *{"$caller\::attribute"} = sub { ref $_[0] ? $_[0] : $_attribute };
+        *{"$caller\::_attributes"} = sub { ref $_[0] ? $_[0] : $_attributes };
+        *{"$caller\::attribute"} = sub { Carp::carp("attribute has been deprecated."); $_[0]->_attributes };
     }
 
     eval "use $schema"; ## no critic
@@ -75,13 +76,25 @@ sub import {
         die $@ if $@ && $@ !~ /Can't locate $schema_file\.pm in \@INC/;
     }
 
+    if ($opt{auto_row_class}) {
+        my $schema_info = $schema->schema_info;
+        for my $table (keys %$schema_info) {
+            my $row_class = join '::', $caller, 'Row', _camelize($table);
+
+            eval "use $row_class"; ## no critic
+            if ($@) { no strict 'refs'; @{"$row_class\::ISA"} = ('DBIx::Skinny::Row') };
+
+            $_attributes->{row_class_map}->{$table} = $row_class;
+        }
+    }
+
     strict->import;
     warnings->import;
 }
 
 sub new {
     my ($class, $connection_info) = @_;
-    my $attr = $class->attribute;
+    my $attr = $class->_attributes;
 
     $attr->{last_pid} = $$;
 
@@ -100,12 +113,12 @@ sub new {
 
     if ($connection_info) {
 
-        $self->attribute->{profiler} = $unstorable_attribute{profiler};
+        $self->_attributes->{profiler} = $unstorable_attribute{profiler};
 
         if ( $connection_info->{on_connect_do} ) {
-            $self->attribute->{on_connect_do} = $connection_info->{on_connect_do};
+            $self->_attributes->{on_connect_do} = $connection_info->{on_connect_do};
         } else {
-            $self->attribute->{on_connect_do} = $unstorable_attribute{on_connect_do};
+            $self->_attributes->{on_connect_do} = $unstorable_attribute{on_connect_do};
         }
 
         if ($connection_info->{dbh}) {
@@ -118,7 +131,7 @@ sub new {
 
     } else {
         for my $key ( keys %unstorable_attribute ) {
-            $self->attribute->{$key} = $unstorable_attribute{$key};
+            $self->_attributes->{$key} = $unstorable_attribute{$key};
         }
     }
 
@@ -127,7 +140,7 @@ sub new {
 
 my $schema_checked = 0;
 sub schema { 
-    my $attribute = $_[0]->attribute;
+    my $attribute = $_[0]->_attributes;
     my $schema = $attribute->{schema};
     if ( $attribute->{check_schema} && !$schema_checked ) {
         do {
@@ -143,7 +156,7 @@ sub schema {
 
 sub profiler {
     my ($class, $sql, $bind) = @_;
-    my $attr = $class->attribute;
+    my $attr = $class->_attributes;
     if ($attr->{profiler} && $sql) {
         $attr->{profiler}->record_query($sql, $bind);
     }
@@ -152,8 +165,8 @@ sub profiler {
 
 sub suppress_row_objects {
     my ($class, $mode) = @_;
-    return $class->attribute->{suppress_row_objects} unless defined $mode;
-    $class->attribute->{suppress_row_objects} = $mode;
+    return $class->_attributes->{suppress_row_objects} unless defined $mode;
+    $class->_attributes->{suppress_row_objects} = $mode;
 }
 
 #--------------------------------------------------------------------------------
@@ -164,47 +177,47 @@ sub txn_scope {
 
 sub txn_begin {
     my $class = shift;
-    return if ( ++$class->attribute->{active_transaction} > 1 );
+    return if ( ++$class->_attributes->{active_transaction} > 1 );
     $class->profiler("BEGIN WORK");
-    eval { $class->dbh->begin_work } or Carp::croak $@;
+    $class->dbh->begin_work;
 }
 
 sub txn_rollback {
     my $class = shift;
-    return unless $class->attribute->{active_transaction};
+    return unless $class->_attributes->{active_transaction};
 
-    if ( $class->attribute->{active_transaction} == 1 ) {
+    if ( $class->_attributes->{active_transaction} == 1 ) {
         $class->profiler("ROLLBACK WORK");
-        eval { $class->dbh->rollback } or Carp::croak $@;
+        $class->dbh->rollback;
         $class->txn_end;
     }
-    elsif ( $class->attribute->{active_transaction} > 1 ) {
-        $class->attribute->{active_transaction}--;
-        $class->attribute->{rollbacked_in_nested_transaction}++;
+    elsif ( $class->_attributes->{active_transaction} > 1 ) {
+        $class->_attributes->{active_transaction}--;
+        $class->_attributes->{rollbacked_in_nested_transaction}++;
     }
 
 }
 
 sub txn_commit {
     my $class = shift;
-    return unless $class->attribute->{active_transaction};
+    return unless $class->_attributes->{active_transaction};
 
-    if ( $class->attribute->{rollbacked_in_nested_transaction} ) {
+    if ( $class->_attributes->{rollbacked_in_nested_transaction} ) {
         Carp::croak "tried to commit but already rollbacked in nested transaction.";
     }
-    elsif ( $class->attribute->{active_transaction} > 1 ) {
-        $class->attribute->{active_transaction}--;
+    elsif ( $class->_attributes->{active_transaction} > 1 ) {
+        $class->_attributes->{active_transaction}--;
         return;
     }
 
     $class->profiler("COMMIT WORK");
-    eval { $class->dbh->commit } or Carp::croak $@;
+    $class->dbh->commit;
     $class->txn_end;
 }
 
 sub txn_end {
-    $_[0]->attribute->{active_transaction} = 0;
-    $_[0]->attribute->{rollbacked_in_nested_transaction} = 0;
+    $_[0]->_attributes->{active_transaction} = 0;
+    $_[0]->_attributes->{rollbacked_in_nested_transaction} = 0;
 }
 
 #--------------------------------------------------------------------------------
@@ -212,7 +225,7 @@ sub txn_end {
 sub connect_info {
     my ($class, $connect_info) = @_;
 
-    my $attr = $class->attribute;
+    my $attr = $class->_attributes;
     $attr->{dsn} = $connect_info->{dsn};
     $attr->{username} = $connect_info->{username};
     $attr->{password} = $connect_info->{password};
@@ -226,7 +239,7 @@ sub connect {
 
     $class->connect_info(@_) if scalar @_ >= 1;
 
-    my $attr = $class->attribute;
+    my $attr = $class->_attributes;
     my $do_connected;
     if ( !$attr->{dbh} ) {
         $do_connected++;
@@ -254,7 +267,7 @@ sub reconnect {
 sub do_on_connect {
     my $class = shift;
 
-    my $on_connect_do = $class->attribute->{on_connect_do};
+    my $on_connect_do = $class->_attributes->{on_connect_do};
     if (not ref($on_connect_do)) {
         $class->do($on_connect_do);
     } elsif (ref($on_connect_do) eq 'CODE') {
@@ -268,25 +281,25 @@ sub do_on_connect {
 
 sub disconnect {
     my $class = shift;
-    $class->attribute->{dbh} = undef;
+    $class->_attributes->{dbh} = undef;
 }
 
 sub set_dbh {
     my ($class, $dbh) = @_;
-    $class->attribute->{dbh} = $dbh;
+    $class->_attributes->{dbh} = $dbh;
     $class->setup_dbd({dbh => $dbh});
 }
 
 sub setup_dbd {
     my ($class, $args) = @_;
     my $dbd_type = _dbd_type($args);
-    $class->attribute->{dbd} = DBIx::Skinny::DBD->new($dbd_type);
+    $class->_attributes->{dbd} = DBIx::Skinny::DBD->new($dbd_type);
 }
 
 sub dbd {
-    $_[0]->attribute->{dbd} or do {
+    $_[0]->_attributes->{dbd} or do {
         require Data::Dumper;
-        Carp::croak("attribute dbd does not exist. does it connected? attribute: @{[ Data::Dumper::Dumper($_[0]->attribute) ]}");
+        Carp::croak("attribute dbd does not exist. does it connected? attribute: @{[ Data::Dumper::Dumper($_[0]->_attributes) ]}");
     };
 }
 
@@ -294,8 +307,8 @@ sub dbh {
     my $class = shift;
 
     my $dbh = $class->connect;
-    if ( $class->attribute->{last_pid} != $$ ) {
-        $class->attribute->{last_pid} = $$;
+    if ( $class->_attributes->{last_pid} != $$ ) {
+        $class->_attributes->{last_pid} = $$;
         $dbh->{InactiveDestroy} = 1;
         $dbh = $class->reconnect;
     }
@@ -452,11 +465,27 @@ sub find_or_new {
     my ($class, $table, $args) = @_;
     my $row = $class->single($table, $args);
     unless ($row) {
-        $row = $class->data2itr($table, [$args])->first;
+        $row = $class->hash_to_row($table, $args);
     }
     return $row;
 }
 
+sub hash_to_row {
+    my ($class, $table, $hash) = @_;
+
+    my $row_class = $class->_mk_row_class($table.$hash, $table);
+    my $row = $row_class->new(
+        {
+            sql            => undef,
+            row_data       => $hash,
+            skinny         => $class,
+            opt_table_info => $table,
+        }
+    );
+    $row->setup;
+    $row;
+}
+
 sub _get_sth_iterator {
     my ($class, $sql, $sth, $opt_table_info) = @_;
 
@@ -488,7 +517,7 @@ sub _mk_anon_row_class {
     my $row_class = 'DBIx::Skinny::Row::C';
     $row_class .= Digest::SHA1::sha1_hex($key);
 
-    my $attr = $class->attribute;
+    my $attr = $class->_attributes;
     $attr->{base_row_class} ||= do {
         my $tmp_base_row_class = join '::', $attr->{klass}, 'Row';
         eval "use $tmp_base_row_class"; ## no critic
@@ -519,7 +548,7 @@ sub _mk_row_class {
     my ($class, $key, $table) = @_;
 
     $table ||= $class->_guess_table_name($key)||'';
-    my $attr = $class->attribute;
+    my $attr = $class->_attributes;
     my $base_row_class = $attr->{row_class_map}->{$table}||'';
 
     if ( $base_row_class eq 'DBIx::Skinny::Row' ) {
@@ -663,7 +692,7 @@ sub replace {
 sub bulk_insert {
     my ($class, $table, $args) = @_;
 
-    my $code = $class->attribute->{dbd}->can('bulk_insert') or Carp::croak "dbd don't provide bulk_insert method";
+    my $code = $class->_attributes->{dbd}->can('bulk_insert') or Carp::croak "dbd don't provide bulk_insert method";
     $code->($class, $table, $args);
 }
 
@@ -1184,6 +1213,17 @@ get transaction scope object.
         $txn->commit;
     }
 
+=item $skinny->hash_to_row($table_name, $row_data_hash_ref)
+
+make DBIx::Skinny::Row's class from hash_ref.
+
+    my $row = Your::Model->hash_to_row('user',
+        {
+            id   => 1,
+            name => 'lestrrat',
+        }
+    );
+
 =item $skinny->data2itr($table_name, \@rows_data)
 
 DBIx::Skinny::Iterator is made based on \@rows_data.
@@ -1284,6 +1324,22 @@ see L<DBIx::Skinny::Profiler::Trace>
 
     $ SKINNY_TRACE=1 perl ./your_script.pl
 
+=head2 TRIGGER
+
+    my $row = $db->insert($args);
+    # pre_insert: ($db, $args, $table_name)
+    # post_insert: ($db, $row, $table_name)
+
+    my $updated_rows_count = $db->update($args);
+    my $updated_rows_count = $row->update(); # example $args: +{ id => $row->id }
+    # pre_update: ($db, $args, $table_name)
+    # post_update: ($db, $updated_rows_count, $table_name)
+
+    my $deleted_rows_count = $db->delete($args);
+    my $deleted_rows_count = $row->delete(); # example $args: +{ id => $row->id }
+    # pre_delete: ($db, $args, $table_name)
+    # post_delete: ($db, $deleted_rows_count, $table_name)
+
 =head1 BUGS AND LIMITATIONS
 
 No bugs have been reported.
