@@ -1,12 +1,14 @@
 package Mock::CustomProfiler;
 use t::Utils;
 use Mock::CustomProfiler::Profiler;
-use DBIx::Skinny setup => +{
-    dsn => 'dbi:SQLite:',
-    username => '',
-    password => '',
+use DBIx::Skinny 
     profiler => Mock::CustomProfiler::Profiler->new,
-};
+    connect_info => +{
+        dsn => 'dbi:SQLite:',
+        username => '',
+        password => '',
+    }
+;
 
 sub setup_test_db {
     shift->do(q{
