@@ -1,5 +1,5 @@
 package Mock::Inclusion;
-use DBIx::Skinny setup => +{
+use DBIx::Skinny connect_info => +{
 };
 
 sub setup_test_db {
